# -*- coding: utf-8 -*-
#############################################################################
#
#    Author: Guewen Baconnier
#    Copyright 2012 Camptocamp SA
#    Copyright 2015 InfoPrimo SL
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

# import openerp.addons.product.product as product
from osv import osv, fields
from tools.translate import _

# import ipdb

# TODO: verificar que un producto no tenga como código a la equivalencia de otro!!!!!!!

def check_ean(eancode):
    """
        Para saltear la validación EAN se usa esta func vacía. Es ejecutada en lugar la del modelo padre.
        Si se va a usar validacion EAN se puede insertar el código aquí.
        Como alternativa se puede hacer lo que sugiere el comentario del `def _check_ean_key`, lin. 110.
    """
    return True



class ProductEan13(osv.osv):

    """ table: product_ean13 """
    _name = 'product.ean13'
    _description = "List of EAN13 for a product."
    _order = 'sequence'

    """ TODO: Si un articulo es desactivado <eliminado> del maestro de productos
    debera preservarse sus codigos a efectos de soportar informes historicos
    de ventas, compras, movimientos de stock, etc. Todas sus equivalencias de
    codigo pasaran al estado 'active = False'.(no, capas no salen en los reportes)"""

    _columns = {
                'name': fields.char('Ean13 code', size=13, help="Se permiten codigos internos no Ean"),
                'product_id': fields.many2one('product.product', 'Product',
                                                required=True),

                'sequence': fields.integer('Sequence', help="If there are equivalent codes then the minor sequence value the major priority."
                                                             "Si el articulo tiene codigos equivalentes, la Secuencia determina la prioridad"
                                                             "de la equivalencia. La menor de las secuencias tiene la prioridad mas alta." ),

                'ean_action': fields.selection([('C', 'Crear'   ),
                                                ('D', 'Eliminar'),
                                                ('N', 'Normal'  )],
                                                 "Estado Pos", required=True,
                     help = "Cada equivalencia de codigo tiene su estado \n" \
                          + " propio e independiente:                    \n" \
                          + "                                            \n" \
                          + "       'C' = Crear                          \n" \
                          + "       'D' = Eliminar                       \n" \
                          + "       'N' = Normal, sin accion prevista.   \n" \
                          + "                                            \n" \
                          + " Un estado indica la accion que se ejecutara\n" \
                          + " la proxima vez que se generen Archivos     \n" \
                          + " para el POS."),
    }

    """
                    El cambio de estado de 'ean_action' afecta al estado del artículo al
                    que apunta el registro! "product_id".

                    El 'on_change' actúa si los cambios son recibidos desde el interfaz
                    de usuario, por lo que un 'workflow' sería más decuado.

                    El punto es que, al recorrer la tabla de artículos para enviar al pos
                    hay que consultar el estado de este campo.

                    Tal vez lo más 'barato' sea simplemente cambiar el estado de este campo y al
                    momento de enviar modificaciones al POS, recorrer la tabla `product.ean13`
                    o sea este objeto.
    """


    _default = {
            'ean_action': 'C',
    }

    """
    def re_sequenciate(self, cr, uid, ids):
        # re secuanciar
        for row in self.browse(cr, uid, ids):
            product = row.product_id

    def guorkflou_para_el_campo_ean_action():
        pass
    """

    def _check_ean_key(self, cr, uid, ids):
        res = False
        for ean in self.browse(cr, uid, ids):
            """ To enable ean validation you need to
                import openerp.addons.product.product as product
                and replace next line with this:
                res = product.check_ean(ean.name)
                """
            res = check_ean(ean.name)
        return res

    _sql_constraints = [('name', 'unique(name)', 'Oops! The code is already in use')]


    def create(self, cr, uid, vals, context=None):
        """Create ean13 with a sequence higher than all
        other products when it is not specified.

            El `vals` debería ser algo así:
            vals = {'name': '123456', 'product_id': 654321,
                    'sequence': 0, 'ean_action': 'C'
            }

        """

        prod_ids   = self.pool.get('product.product').search(cr, uid, [])
        all_codes  = self.pool.get('product.product').read(cr,uid,prod_ids,['cod_sto'])

        for code in all_codes:
            if vals['name'] == code['cod_sto']:
                msg = _("The equivalent code is already being used!")
                if vals['product_id'] == code['id']:
                    msg = _("The equivalent should be different from the product code!")
                raise osv.except_osv( _("Create POS Error"), msg )


        if not vals.get('sequence') and vals.get('product_id'):
            ean13_ids = self.search(
                cr, uid,
                [('product_id', '=', vals['product_id'])],
                context=context)
            ean13s = self.browse(cr, uid, ean13_ids, context=context)
            vals['sequence'] = 1
            vals['ean_action'] = 'C'
            if ean13s:
                vals['sequence'] = max([ean.sequence for ean in ean13s]) + 1
        result = super(ProductEan13, self).create(cr, uid, vals, context=context)
        if result:
            self.pool.get('product.product').write(cr,uid,vals['product_id'],{'pos_action': 'I'})

        return result

ProductEan13()



class ProductProductEan(osv.osv):


    """ Inherit the model product.product in order to create the m2o link
        This class definition has to be before the others below to ensure that
        the table is created before the migration

    El único propósito de esta clase python es provocar la herencia openerp durante la primera actualización del módulo
    "ip_poduct"
    que el nombre de la calse Python sea diferente al de model openerp no complica porque para open el nombre de la "clase"
    es `_name` . En tando que el monbre de la isntancia `ProductProductEan()` será algo que quede boyando en el namespace
    sin que nadie le de pelota.
    """
    _inherit = 'product.product'

    _columns = {
        'ean13_ids': fields.one2many('product.ean13', 'product_id', 'Ean13', required=True, ondelete='cascade'),
    }



ProductProductEan()

"""
class ProductProductMigration(osv.osv):
     Inherit the model product.product in order to migrate the ean13
        column in the m2o table product_ean13
        This class definition has to be before the others below to ensure that
        the migration is done before the creation of the ean13 fields.function
    " ""

    _inherit = 'product.product'

    _columns = {
        # create 'legacy_ean13' field by copying the contents of field 'ean13' (oldname)
        'legacy_ean13': fields.char('Original EAN13', size=13, oldname='ean13'),
        }

    # transfer the 'legacy_ean13' field to other table.
    def init(self, cr):
        cr.execute("INSERT INTO "
                   "product_ean13 (name, product_id, sequence) "
                   "SELECT p.legacy_ean13, p.id, 1 "
                   "FROM product_product p WHERE "
                   "legacy_ean13 IS NOT NULL "
                   "AND NOT EXISTS "
                   "(SELECT id FROM product_ean13 "
                   " WHERE name = p.legacy_ean13 "
                   "       AND product_id = p.id)")

ProductProductMigration()
"""


class ProductProduct(osv.osv):

    """
        Acá el propósito es agregar el campo fucional 'ean13'

    """

    _inherit = 'product.product'

    def _get_main_ean13(self, cr, uid, ids, field_name, arg, context):
        values = {}
        for product in self.browse(cr, uid, ids, context):
            ean13 = False
            if product.ean13_ids:
                # get the first ean13 as main ean13
                ean13 = product.ean13_ids[0].id
            values[product.id] = ean13
        return values

    def _get_ean(self, cr, uid, ids, context=None):
        res = set()
        obj = self.pool.get('product.ean13')
        for ean in obj.browse(cr, uid, ids, context):
            res.add(ean.product_id.id)
        return list(res)

    def _write_ean(self, cr, uid, product_id, name, value, arg, context=None):

        product = self.browse(cr, uid, product_id, context=context)

        if not value in [ean.name for ean in product.ean13_ids]:
            self.pool.get('product.ean13').create(
                cr, uid,
                {'name': value, 'product_id': product.id, 'ean_action':'C' },
                context=context)
            try:
                self.pool.get('product.product').write(cr,uid,product_id,{'pos_action': 'I'})
            except:
                raise osv.except_osv("¿Que pasó?", "Pasó que en _write_ean -> self.pool.get('product.product').write(cr,uid,product_id,{'pos_action': 'I'}) falló")
                #ipdb.set_trace()
        return True

    _columns = {

        'ean13': fields.function( _get_main_ean13,
                                  fnct_inv = _write_ean,
                                  type = 'many2one',
                                  obj = 'product.ean13',
                                  store = {
                                    'product.product': (lambda self, cr, uid, ids, c=None: ids, ['ean13_ids'], 10),
                                    'product.ean13': (_get_ean, [], 10)},
                                  string='Main EAN13',
                                  method=True,
                                  readonly=True)

    }

    # disable constraint
    def _check_ean_key(self, cr, uid, ids):
        "Inherit the method to enable the EAN13 check"
        return True
    _constraints = [(_check_ean_key, 'Error: Invalid ean code', ['ean13'])]

    def search(self, cr, uid, args, offset=0, limit=None,
               order=None, context=None, count=False):
        """overwrite the search method in order to search
        on all ean13 codes of a product when we search an ean13"""

        if filter(lambda x: x[0] == 'ean13', args):
            # get the operator of the search
            ean_operator = filter(lambda x: x[0] == 'ean13', args)[0][1]
            #get the value of the search
            ean_value = filter(lambda x: x[0] == 'ean13', args)[0][2]
            # search the ean13
            ean_ids = self.pool.get('product.ean13').search(
                cr, uid, [('name', ean_operator, ean_value)])

            #get the other arguments of the search
            args = filter(lambda x: x[0] != 'ean13', args)
            #add the new criterion
            args += [('ean13_ids', 'in', ean_ids)]
        # print 'args : %s \noffset : %s ' % (args, offset)
        return super(ProductProduct, self).search(
            cr, uid, args, offset, limit, order, context=context, count=count)

ProductProduct()
