#  -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#    Copyright (C) 2010-2012 Camptocamp Austria (<http://www.camptocamp.at>)
#    Copyright (C) 2015 Infoprimo SL (<http://infoprimo.com.uy>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import openerp.addons.product.product as product

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp
import time

import pydevd

class product_template(osv.osv):
    """
        Campos añadidos son campos utiles de la antigua estructura de
        archivos dbf. El codigo de articulo viene en “cod_sto”, donde era
        llave primaria.

        El contenido de “cod_aso” en un codigo de articulo valido y referencia
        a un articulo, “cod_sto”, de la misma tabla.

        Se usaba para asociciar articulos. Un ejemplo de venta sería el de los
        articulos con envase retornable, luego de importar datos de los dbf.
        En este caso el articulo tendra la referencia al envase asociado en
        el campo “cod_aso”.

        En openerp se uso una ralacion many2one desde “product.template” a
        “product.product”.

        Ver: 'asoc_code_id': fields.many2one('product.product'),  ...
    """

    _inherit = "product.template"

    def _check_markup(self, cr, uid, ids, context=None):
        """ markup > 0 """
        context = context or {}
        record = self.browse(cr, uid, ids, context=context)
        for data in record:
            if data.margen < 0:
                return False
        return True

    _columns = {
        # columnas migradas
        # este campo sustituyó a “default_code” y lo mismo se debe hacer con “cod_aso”
        'cod_sto': fields.char('Codigo', size=13, required=True, select=True),
        'sto_pro': fields.char(': Cod. del proveedor', size=20),
        'cod_pro': fields.char(': Num. del proveedor', size=4),
        'cod_sub': fields.char(': Num. subfamilia', size=3),
        'nom_sto': fields.char(': Nombre', size=128),
        'pre_sto': fields.float('Precio de Venta', digits_compute=dp.get_precision('Purchase Price')),
        'iva_sto': fields.char(': Iva', size=4, help="El ERP aún está en migración. En el sistema anterior «stock», este valor significaba 1: Exentos | 2: T.Min | 3: T.Max \n"""
        "En el ERP ese criterio ya no se usa.    Ahora el IVA del artículo se fija en la solapa 'Contabilidad'.\nSin embargo, este valor debe ser asignado a la antigua, es decir con el criterio del sistema anterior «stock», hasta tanto la migración de datos no esté terminada.",
                required=True),
        'margen' : fields.float('Margen bruto', help="[ Debe ser >= a 0 ]", digits_compute=dp.get_precision('Purchase Price'), required=True),
        'last_purchase_price': fields.float('Precio Ultima Compra iva incl.', digits_compute=dp.get_precision('Purchase Price')),
        'subfam_id': fields.many2one('product.category','Subfam Ref', change_default=True, domain="[('categoria','>',19)]" ),
        # ver 'asoc_code_id' más abajo.
        'cod_aso': fields.char('Codigo asociado', size=13),
        'opcion1': fields.char(': Opcion1', size=13),
        'description_sale': fields.char('Sale Description', size=32, required=True),
        'description_purchase': fields.text('Purchase Description',translate=True, required=True),

        'description': fields.text('Description',translate=True, required=True),

        # columnas agregadas
        # agregado 'real' al selection
        'cost_method': fields.selection([('real', 'Real Cost'), ('standard','Standard Price'), ('average','Average Price')], 'Costing Method', required=True),

        # También se usan del ancestro:
        # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

        # Base price for computing the customer price. Sometimes called the catalog price.

        #'list_price': fields.float('Sale Price', digits_compute=dp.get_precision('Sale Price'), help="Base price for computing the customer price. Sometimes called the catalog price."),

        #'list_price': fields.float('Sale Price', digits_compute=dp.get_precision('Sale Price'), help="Base price for computing the customer price. Sometimes called the catalog price."),

        #'standard_price': fields.float('Cost Price', required=True, digits_compute=dp.get_precision('Purchase Price'), help="Product's cost for accounting stock valuation. It is the base price for the supplier price."),
        'old_id'    : fields.integer('id en la DB anterior', readonly=True,
                                     help="El propósito es soportar mejor algunas operaciones "
                                          "de migració. Facilita reenlazar entidades, etc"),
    }

    _default = {
        'margen': 0.1,
        'iva_sto': '3',
    }

    _constraints = [(_check_markup, 'Error: markup must be >= zero', ['margen'])]



product_template()


class product_product(osv.osv):

    """
        En la extencion se crea el campo 'acction_pos'
        que indica la accion que se enviará al POS
        en la siguiente sincronizacion.

        ATENCION: La view de este campo fue a parar al
        archivo: 'multi_ean/product_ean_view.xml'
    """
    _inherit = 'product.product'

    _columns = {
        'trial_price': fields.float('Trial', digits_compute=dp.get_precision('Sale Price'), help="Precio de venta al Publico"),
        'real_cost': fields.float('Real Price', digits_compute=dp.get_precision('Purchase Price'), help="Product's cost from last purchase. It is the supplier price."),
        'taxset':   fields.boolean('Pruduct must be tax assigned. Please check you entry.', required=True),
        #'trial_price': fields.funcion(_get_trial_price, method=True, store=True, string='Trial', type="float", digits_compute=dp.get_precision('Sale Price'), help="Precio de venta al Público"),
        #'real_cost': fields.function (_get_real_price,  method=True, store=True, tring='Real Price',type="float", digits_compute=dp.get_precision('Purchase Price'), help="Product's cost from last purchase. It is the supplier price."),

        'asoc_code_id': fields.many2one('product.product', string='Envase asociado', help="Asociar el envase si corresponde."),
        'envase_obligatorio': fields.boolean('Envase obligatorio', help="El articulo debe llevar envase asociado"),
        'es_envase': fields.boolean('Envase', help="Si esta marcado, el articulo es un envase retornable"),

        'pos_action': fields.selection([('I', 'Insertar'),
                                        ('U', 'Actualizar'),
                                        ('D', 'Eliminar'),
                                        ('A', 'Insertar/Actualizar (Batch)'),
                                        ('X', 'Borrado Parcial (Batch)'),
                                        ('Z', 'Borrado Total (Batch)'),
                                        ('N', 'Sin accion'  )],
                                        "Accion en el POS", required=True,
                                        help="Es la accion a ejecutarse la proxima vez que se generaren los archivos para el POS."),
        # 'description_sale': fields.text('Sale Description',translate=True, required=True),
        'in_pos': fields.boolean('The product already existe in POS database'),
    }

    _defaults = {
        'envase_obligatorio': False,
        'es_envase': False,
        'pos_action': 'I', # al crearse el artículo debe ir al POS
        'taxset': False,
        'in_pos': False,
    }
    """
     TODO: Es necesario definir un workflow para el estado respecto al POS
           inicia en 'C' 'Insertar', una vez enviado al POS, debe pasar a 'N'
           Sin accion'. Luego, cualquier cambio en los campos del articulo
           que sean actualizables en el POS deberan poner la bandera en 'U'
           'Actualizar'
           1. hacer la lista de campos que sean actualizables en el POS
    """
    def guorkflou_para_el_campo_pos_action(self):
        pass

    def change_iva_sto(self):
        res = {}
        return res

    def ch_cod_sto(self, cr, uid, ids, cod_sto, context=None):
        res = {}
        if not ids or not cod_sto:
            return res
        context = context or {}
        cod_sto = cod_sto.replace(' ','').strip()
        vals = { 'cod_sto': cod_sto }
        warn = {}
        if not cod_sto.isdigit() or cod_sto[0] is '0':
            vals = { 'cod_sto': False }
            msg = "Code %s is invalid, please use only digits!" % cod_sto
            warn = { 'title': _('Warning'), 'message': _( msg ) }
        res =  { 'value': vals, 'warning': warn }
        return res

    def write(self, cr, uid, ids, vals, context=None):

        # ipdb.set_trace()

        res = {}
        context = context or {}

        if 'cod_sto' in vals.keys():
            if not vals['cod_sto'].isdigit():
                msg = "Code %s is invalid, please use only digits!" % (vals['cod_sto'],)
                raise osv.except_osv('Error', _( msg) )
            else:
                # compatibilidad frente a otros usos de 'default_code'
                vals['default_code'] = vals['cod_sto']

        res = super(product_product,self).write(cr, uid, ids, vals, context)
        return res

    def _check_default_code(self, cr, uid, ids, context=None):
        """
        Verify `cod_sto` and `default_code` are the same.
        :param cr, uid, ids, context:    parametros openerp por defecto
        :return: True of False
        """
        # import ipdb;ipdb.set_trace()
        if not ids: return {}
        context = context or {}
        res = False
        prod = self.browse(cr, uid, ids[0], context)
        pydevd.settrace("XPS-15-9550", port=5678)
        print("la pija {}".format('boba', ))

        return True

    check_default_code_msg = "El campo `default_code` difiere de `cod_sto`. Por favor, avise al administrador"
    _constraints = [( _check_default_code, check_default_code_msg, ['default_code'])]

    def _check_self_equiv(self, cr, uid, ids, context=None):
        """
        Ensure the `cod_sto` of just created product is present as equivalence for this product
        :param cr, uid, ids, context:    parametros openerp por defecto
        :return: True of False
        """
        if not ids: return {}
        context = context or {}
        res = False
        prod = self.browse(cr, uid, ids[0], context)

        return True

    check_self_equiv_msg = "La equivalencia principal del producto no existe. Por favor, avise al administrador"
    _constraints = [( _check_self_equiv, check_self_equiv_msg, ['default_code'])]

    def _check_envase_obligatorio(self, cr, uid, ids, context=None):
        """
            If the product has obligatory returnable
            then 'asoc_code_id' must not be empty.
        """

        if not ids: return {}
        context = context or {}
        res = False
        prod = self.browse(cr, uid, ids[0], context)
        if prod.envase_obligatorio and prod.asoc_code_id:
            res = True
        return res

    envase_obligatorio_msg = "Envase obligatorio! Por favor, asocia el envase!"
    _constraints = [(_check_envase_obligatorio, envase_obligatorio_msg, ['envase_obligatorio'])]

    def _check_es_envase(self, cr, uid, ids, context=None):
        """
            If the product is a returnable then
            'asoc_code_id' must be empty.
            We don't like recurssion here.
        """

        if not ids: return {}
        context = context or {}
        res = True
        prod = self.browse(cr, uid, ids[0], context)
        if prod.es_envase and prod.asoc_code_id:
            print '---->', prod.id, prod.cod_sto, prod.name
            res = False
        return res

    es_envase_msg = "El articulo es un envase!. Verificar que el codigo asociado este vacio codigo."
    _constraints = [(_check_es_envase, es_envase_msg, ['es_envase'])]

    def _taxset(self,cr,uid,ids):
        if not ids: return True
        res = False
        for prod in self.browse(cr,uid,ids):
            if prod.taxes_id and prod.supplier_taxes_id:
                return True
        return False

    taxset_msg = _("Please choose the product tax.")
    _constraints = [(_taxset, taxset_msg, ['taxset'])]



    def _chk_categ(self,cr,uid,ids):
        if not ids: return True
        res = False
        for prod in self.browse(cr,uid,ids):
            if prod.categ_id > 19:
                return True
        return False

    categ_msg = _("La categoría debe ser mayor que 19")
    _constraints = [(_chk_categ, categ_msg, ['categ_id'])]



    def ch_asoc(self, cr, uid, ids, asoc_code_id, es_envase=False, envase_obligatorio=False, context=None ):
        # ipdb.set_trace()
        if not ids: return {}
        context = context or {}
        res = {}
        prod = self.browse(cr, uid, ids[0])

        # The product can't be self contaier
        if prod.id == asoc_code_id:
            msg = " %s : es envase de si mismo. \n" % prod.name
            msg += " Por favor, varifica antes de guardar! "
            msgtr = _(msg)
            raise osv.except_osv('Error !', msg)

        # If not empty "asoc_code_id" is incoming
        # then "envase_obligatorio" must be True
        if asoc_code_id:
            #ipdb.set_trace()
            if not self.browse(cr, uid, [asoc_code_id,])[0].es_envase:
                msg = _("El articulo que intenta asociar no es un envase.")
                raise osv.except_osv('Error !', msg )
        return res


product_product()


class product_historical(osv.osv):
    """
    product_historical
    """
    _inherit = 'product.product'

    def _get_historical_price_FFM(self, cr, uid, ids, field_name, field_value, arg, context=None):

        if context is None:
            context = {}
        res = {}
        product_hist = self.pool.get('product.historic.price')
        for id in ids:
            if self.browse(cr, uid, id).list_price != self.browse(cr, uid, id).list_price_historical:
                res[id] = self.browse(cr, uid, id).list_price
                product_hist.create(cr, uid, {
                    'product_id' : id,
                    'name': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'price': self.browse(cr, uid, id).list_price,
               },context)
        return res

    def _get_historical_cost_FFM(self, cr, uid, ids, field_name, field_value, arg, context=None):
        if context is None:
            context = {}
        res = {}
        product_hist = self.pool.get('product.historic.cost')
        for id in ids:
            if self.browse(cr, uid, id).standard_price != self.browse(cr, uid, id).list_cost_historical:
                res[id] = self.browse(cr, uid, id).standard_price
                product_hist.create(cr, uid, {
                    'product_id' : id,
                    'name': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'price': self.browse(cr, uid, id).standard_price,
               },context)
        return res

    _columns = {
        'list_price_historical': fields.function(_get_historical_price_FFM, method=True, string='Latest Price', type='float',digits_compute= dp.get_precision('List_Price_Historical'),
            store={'product.product': (lambda self, cr, uid, ids, c={}: ids, ['list_price'], 50),},
            help="Latest Recorded Historical Value"),

        'list_cost_historical': fields.function(_get_historical_cost_FFM, method=True, string=' Latest Cost', type='float',digits_compute= dp.get_precision('Cost_Historical'),
            store={'product.product': (lambda self, cr, uid, ids, c={}: ids, ['standard_price'], 50),},
            help="Latest Recorded Historical Cost"),

        'list_price_historical_ids': fields.one2many('product.historic.price','product_id','Historical Prices' ),
        'list_cost_historical_ids': fields.one2many('product.historic.cost','product_id','Historical Prices' ),
    }


product_historical()


class product_historic_price(osv.osv):
    """
        Implements the objet which represents
        the historic sale prices list
    """
    _order= "name desc"
    _name = "product.historic.price"
    _description = "Historical Public Price List"

    _columns = {
        'product_id': fields.many2one('product.product',string='Product related to this Price', required=True, ondelete='cascade'),
        'name': fields.datetime(string='Date',  required=True),
        'price': fields.float(string='Price',digits_compute= dp.get_precision('Price')),
        'product_uom': fields.many2one('product.uom', string="Supplier UoM", help="Choose here the Unit of Measure in which the prices and quantities are expressed below.")
    }
    _defaults = { 'name': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
    }


product_historic_price()


class product_historic_cost(osv.osv):
    """
        Implements the objet which represents
        the historic sale prices list
    """
    _order= "name desc"
    _name = "product.historic.cost"
    _description = "Historical Purchase Price List"

    _columns = {
        'product_id': fields.many2one('product.product', string='Product related to this Cost', required=True, ondelete='cascade'),
        'name': fields.datetime(string='Date', required=True),
        'price': fields.float(string='Cost', digits_compute= dp.get_precision('Price2')),
        'product_uom': fields.many2one('product.uom', string="Supplier UoM", help="Choose here the Unit of Measure in which the prices and quantities are expressed below.")

    }
    _defaults = { 'name': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
    }


product_historic_cost()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
