#!/usr/bin/python
#  -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2015 Infoprimo SL (<http://infoprimo.com.uy>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp
import time
import re

HX = re.compile(r'[0-9A-Fa-f]{16}')
def check_16hex(atr16hex='0000000000000000'):
    atr16hex = atr16hex[:16]
    if not isinstance(atr16hex, str):
        atr16hex='0000000000000000'
    return atr16hex if HX.match(atr16hex) else False


class product_product(osv.osv):

    """ class inherit for POS fields extension. """
    _name        = 'product.product'
    _inherit     = 'product.product'

    def get_atr16hex(self,cr,uid,ids,context=None):
        """
          Debera ser un stored campo funcional, de modo que
          pueda ser calculado y desplegado directamente en runtime.

          Como esta, se calcula, se guarda, se recupera, etc..
          cuando en realidad, depende de los 64 munecos.

          El costo por calculo en cada recorrida de "envios al pos"
          se reduce a los marcados en "pos_action".
        """

        if not ids: return {}
        context = context or {}
        res = {}

        for prod in self.browse(cr,uid,ids,context):
            res[prod] = check_16hex(atr16hex) # verifica que sean 16 caracteres hexa validos
            if not res[prod]:
                msg = 'Attribute "%s" in product "%s" does not seems a hex16 string. Please fix!' % (atr16hex, prod.name)
                raise osv.except_osv( _('Error !'), _( msg ) )
        return res or False

    def _bin2hex(self, cr, uid, ids, field_name, arg, context):

        """ Calculate 'atr16hex' """
        values = {}
        for p in self.browse(cr, uid, ids, context):

            bits = (p.b01, p.b02, p.b03, p.b04, p.b05, p.b06, p.b07, p.b08,
                    p.b09, p.b10, p.b11, p.b12, p.b13, p.b14, p.b15, p.b16,
                    p.b17, p.b18, p.b19, p.b20, p.b21, p.b22, p.b23, p.b24,
                    p.b25, p.b26, p.b27, p.b28, p.b29, p.b30, p.b31, p.b32,
                    p.b33, p.b34, p.b35, p.b36, p.b37, p.b38, p.b39, p.b40,
                    p.b41, p.b42, p.b43, p.b44, p.b45, p.b46, p.b47, p.b48,
                    p.b49, p.b50, p.b51, p.b52, p.b53, p.b54, p.b55, p.b56,
                    p.b57, p.b58, p.b59, p.b60, p.b61, p.b62, p.b63, p.b64)

            qrt = pos_attribs = ''
            for i in range(64):
                qrt += ''.join('1' if bits[i] else '0')
                if (i % 4) == 3:
                    pos_attribs += ''.join(hex(int(qrt,2)).replace('0x','',1))
                    qrt = ''
            values[p.id] = pos_attribs # 'C00C000100000000'
        return values

    _columns = {

        'atr16hex': fields.char('Monitor', size=16, readonly=True,
                                help="Se usa para verificar los cambios de atributos, en formato hexadecimal, a medida que se hacen"),
        'pos_hexatt': fields.function(_bin2hex, type = 'char',
                                      string='Atributos activos',
                                      help="Indica los atributos del artículo que se encuentran activos en el POS en formato hexadecimal",
                                      method=True),

        'b01': fields.boolean(' 1 Comestible.',            required=True, help="El articulo tiene la propiedad de comestible."),
        'b02': fields.boolean(' 2 No Agrupable.',          required=True, help="El ticket detalla la cantidad desglosada para cada escaneo del articulo."),
        'b03': fields.boolean(' 3 Impresion destacada.',   required=True, help="Articulo configurado para impresion destacada en el ticket - negrita."),
        'b04': fields.boolean(' 4 Service garantia.',      required=True, help="Articulo con service de garantia."),
        'b05': fields.boolean(' 5 Puntos/Millas extra.',   required=True, help="El articulo da puntos/millas extras a clientes habilitados."),
        'b06': fields.boolean(' 6 Articulo canjeable.',    required=True),
        'b07': fields.boolean(' 7 Puede ser gratuito.',    required=True),
        'b08': fields.boolean(' 8 - No habilitado'        ,readonly=True, required=True),
        'b09': fields.boolean(' 9 Tandem',                 required=True, help="Tiene articulo asociado/envase"),
        'b10': fields.boolean('10 - No habilitado'        ,readonly=True, required=True),
        'b11': fields.boolean('11 Solo cantidad entera.',  required=True, help="Sin decimales. Si se digita la cantidad de articulos antes del codigo, solo se admitira cantidades enteras. Ej. 3 x Codigo Articulo."),
        'b12': fields.boolean('12 Desempaquetable.',       required=True, help="El codigo del articulo corresponde a un paquete que puede ser vendido por unidades. Ejem. Pack de jabones. Cuando se ingresa un codigo de articulo se pedira la cantidad de items internos del paquete"),
        'b13': fields.boolean('13 Cantidad obligatoria.',  required=True, help="Uso en venta por escalas. El articulo puede requerir el ingreso de la cantidad en forma obligatoria. Es similar a lo anterior, una vez pasado el codigo del articulo, pide la cantidad de articulos total."),
        'b14': fields.boolean('14 Cantidad unitaria.',     required=True, help="El articulo utiliza cantidad unitaria para promociones. Generalmente aplicable a articulos de balanza fraccionados que requieren de cantidad unitaria para poder realizar el  calculo del beneficio."),
        'b15': fields.boolean('15 Precio por escala.',     required=True, help="El articulo utiliza precio por escalas."),
        'b16': fields.boolean('16 Datos adicionales.',     required=True, help="El articulo pide datos adicionales al operador."),
        'b17': fields.boolean('17 Descuento sub-familia',  required=True, help="Solo aplica el descuento de la sub-familia."),
        'b18': fields.boolean('18 - No habilitado'        ,readonly=True, required=True),
        'b19': fields.boolean('19 Precio de entorno.',     required=True, help="El precio se aplica a todos los entornos."),
        'b20': fields.boolean('20 Venta en entorno',       required=True, help="Puede ser vendido en todos los entornos."),
        'b21': fields.boolean('21 Entorno por defecto',    required=True, help="El producto solo puede ser vendido en el entorno predeterminado."),
        'b22': fields.boolean('22 Entornos multiples',     required=True, help="Puede ser vendido en el entorno predeterminado  y en otros."),
        'b23': fields.boolean('23 - No habilitado'        ,readonly=True, required=True),
        'b24': fields.boolean('24 Precio obligatorio.',    required=True, help="El articulo tiene precio abierto. Puede ser modificado por la cajera."),
        'b25': fields.boolean('25 Venta supervisada.',     required=True, help="Requiere supervisor para la venta."),
        'b26': fields.boolean('26 - No habilitado'        ,readonly=True, required=True),
        'b27': fields.boolean('27 Otra moneda.',           required=True, help="El precio esta en un moneda diferente a la de uso corriente."),
        'b28': fields.boolean('28 - No habilitado'        ,readonly=True, required=True),
        'b29': fields.boolean('29 Inactivo',               required=True, help="No permite su venta."),
        'b30': fields.boolean('30 - No habilitado'        ,readonly=True, required=True),
        'b31': fields.boolean('31 No descontable',         required=True, help="No permite ningun tipo de descuento ni forma parte de la linea para un descuento al total."),
        'b32': fields.boolean('32 Linea al desc.total.',   required=True, help="XXX Forma parte como linea al descuento al total."),
        'b33': fields.boolean('33 - No habilitado'        ,readonly=True, required=True),
        'b34': fields.boolean('34 - No habilitado'        ,readonly=True, required=True),
        'b35': fields.boolean('35 - No habilitado'        ,readonly=True, required=True),
        'b36': fields.boolean('36 Beneficio en la linea.', required=True, help="El beneficio aplica al total de la linea Se toma en cuenta el monto de la linea -cantidad de articulo * monto articulo- para la generacion de chances en sorteos si la suma de los montos de los articulos marcados superan un monto pre-establecido en Parametros Generales - Monto Minimo Chance"),
        'b37': fields.boolean('37 - No habilitado'        ,readonly=True, required=True),
        'b38': fields.boolean('38 Se solicita talle.',     required=True),
        'b39': fields.boolean('39 Se solicita color.',     required=True),
        'b40': fields.boolean('40 Se solicita marca.',     required=True),
        'b41': fields.boolean('41 Se solicita modelo.',    required=True),
        'b42': fields.boolean('42 - No habilitado'        ,readonly=True, required=True),
        'b43': fields.boolean('43 - No habilitado'        ,readonly=True, required=True),
        'b44': fields.boolean('44 - No habilitado'        ,readonly=True, required=True),
        'b45': fields.boolean('45 Alta temporal.',         required=True, help="El articulo fue dado de alta en forma temporal."),
        'b46': fields.boolean('46 Recarga de celular.',    required=True, help="El articulo es una recarga de celular."),
        'b47': fields.boolean('47 - No habilitado'        ,readonly=True, required=True),
        'b48': fields.boolean('48 - No habilitado'        ,readonly=True, required=True),
        'b49': fields.boolean('49 - No habilitado'        ,readonly=True, required=True),
        'b50': fields.boolean('50 - No habilitado'        ,readonly=True, required=True),
        'b51': fields.boolean('51 - No habilitado'        ,readonly=True, required=True),
        'b52': fields.boolean('52 - No habilitado'        ,readonly=True, required=True),
        'b53': fields.boolean('53 - No habilitado'        ,readonly=True, required=True),
        'b54': fields.boolean('54 - No habilitado'        ,readonly=True, required=True),
        'b55': fields.boolean('55 - No habilitado'        ,readonly=True, required=True),
        'b56': fields.boolean('56 - No habilitado'        ,readonly=True, required=True),
        'b57': fields.boolean('57 - No habilitado'        ,readonly=True, required=True),
        'b58': fields.boolean('58 - No habilitado'        ,readonly=True, required=True),
        'b59': fields.boolean('59 - No habilitado'        ,readonly=True, required=True),
        'b60': fields.boolean('60 - No habilitado'        ,readonly=True, required=True),
        'b61': fields.boolean('61 - No habilitado'        ,readonly=True, required=True),
        'b62': fields.boolean('62 - No habilitado'        ,readonly=True, required=True),
        'b63': fields.boolean('63 - No habilitado'        ,readonly=True, required=True),
        'b64': fields.boolean('64 - No habilitado'        ,readonly=True, required=True),
    }

    _defaults = {
                'atr16hex':  lambda *a: '0000000000000000',
                'b01': False, 'b02': False, 'b03': False, 'b04': False,
                'b05': False, 'b06': False, 'b07': False, 'b08': False,
                'b09': False, 'b10': False, 'b11': False, 'b12': False,
                'b13': False, 'b14': False, 'b15': False, 'b16': False,
                'b17': False, 'b18': False, 'b19': False, 'b20': False,
                'b21': False, 'b22': False, 'b23': False, 'b24': False,
                'b25': False, 'b26': False, 'b27': False, 'b28': False,
                'b29': False, 'b30': False, 'b31': False, 'b32': False,
                'b33': False, 'b34': False, 'b35': False, 'b36': False,
                'b37': False, 'b38': False, 'b39': False, 'b40': False,
                'b41': False, 'b42': False, 'b43': False, 'b44': False,
                'b45': False, 'b46': False, 'b47': False, 'b48': False,
                'b49': False, 'b50': False, 'b51': False, 'b52': False,
                'b53': False, 'b54': False, 'b55': False, 'b56': False,
                'b57': False, 'b58': False, 'b59': False, 'b60': False,
                'b61': False, 'b62': False, 'b63': False, 'b64': False,
    }

    def chaAtr(self, cr, uid, ids, atr16hex='0000000000000000',
                b01=0,b02=0,b03=0,b04=0,b05=0,b06=0,b07=0,b08=0,
                b09=0,b10=0,b11=0,b12=0,b13=0,b14=0,b15=0,b16=0,
                b17=0,b18=0,b19=0,b20=0,b21=0,b22=0,b23=0,b24=0,
                b25=0,b26=0,b27=0,b28=0,b29=0,b30=0,b31=0,b32=0,
                b33=0,b34=0,b35=0,b36=0,b37=0,b38=0,b39=0,b40=0,
                b41=0,b42=0,b43=0,b44=0,b45=0,b46=0,b47=0,b48=0,
                b49=0,b50=0,b51=0,b52=0,b53=0,b54=0,b55=0,b56=0,
                b57=0,b58=0,b59=0,b60=0,b61=0,b62=0,b63=0,b64=0,
                context=None):

        if not ids:
            return {}
        context = context or {}

        if not check_16hex(atr16hex):
            prod_name = self.browse(cr,uid,ids[0],context).name
            msg = 'Attribute "%s" in product "%s" does not seems a hex16 string. Please fix!' % (atr16hex, prod.name)
            raise osv.except_osv( _('Error !'), _( msg ) )

        bits = (b01, b02, b03, b04, b05, b06, b07, b08,
                b09, b10, b11, b12, b13, b14, b15, b16,
                b17, b18, b19, b20, b21, b22, b23, b24,
                b25, b26, b27, b28, b29, b30, b31, b32,
                b33, b34, b35, b36, b37, b38, b39, b40,
                b41, b42, b43, b44, b45, b46, b47, b48,
                b49, b50, b51, b52, b53, b54, b55, b56,
                b57, b58, b59, b60, b61, b62, b63, b64)

        # print bits
        res = {}
        hex_attrib = ''
        qrt = ''

        for i in range(64):
            # agrupa los bits en 16 "cuartetos" (16 grupos de medio byte)
            qrt += ''.join('1' if bits[i] else '0')
            if (i % 4) == 3:
                # convierte a hexa el valor de cada "cuarteto"
                hex_attrib += ''.join(hex(int(qrt,2)).replace('0x','',1))
                qrt = ''

        if check_16hex(hex_attrib):
            res = {'value': { 'atr16hex': hex_attrib } }
            if atr16hex != hex_attrib: # mark product as touched
                res['value'].update({'pos_action': 'I' })
        else:
            res = {}
            msg = 'Attribute "%s" or "%s" could be broken. Please check!' % (atr16hex,hex_attrib)
            raise osv.except_osv( _('Error !'), _( msg ) )
        return res



    def write(self, cr, uid, ids, vals, context=None):


        if not ids:
            return {}
        context = context or {}
        res = super(product_product,self).write(cr, uid, ids, vals, context)
        if vals.keys() == ['atr16hex']:
            return res

        if isinstance(ids, (int, long)):
            IDs = '('+str(ids)+')'
        elif isinstance(ids, (list)):
            IDs = '('+str(ids[0])+')' if len(ids) else False

        if not IDs:
            # ALERTA con mensaje si el proceso se hace automático!!
            msg = "Hay un objeto ID que no es ni int, ni long!!"
            osv.except_osv( _('Error !'), _( msg ) )


        qry_traer = '''select b01, b02, b03, b04, b05, b06, b07, b08, b09, b10, b11, b12, b13, b14, b15, b16,b17, b18, b19, b20, b21, b22, b23, b24,b25, b26, b27, b28, b29, b30, b31, b32,b33, b34, b35, b36, b37, b38, b39, b40, b41, b42, b43, b44, b45, b46, b47, b48,b49, b50, b51, b52, b53, b54, b55, b56,b57, b58, b59, b60, b61, b62, b63, b64, id,atr16hex from product_product WHERE id in %s;''' % IDs

        cr.execute(qry_traer)
        recs = cr.fetchall()

        for i in recs:
            atr16hex = check_16hex(str(i[-1:][0]))
            id = i[-2:][0]
            bits = i[:-2]
            b64 = ''
            for bit in bits:
                b64 += ''.join('1' if bit else '0')
            hex_attrib = check_16hex(hex(int(b64, 2)).replace('0x', '', 1))

            if ( not atr16hex and hex_attrib ) or hex_attrib:
                    cr.execute('''update product_product set atr16hex = '%s' where id = %s;''' % (hex_attrib,id))
        return res


product_product()

