# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright Camptocamp SA, InfoPrimo SL 2015,
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

#import ipdb
#import pickle
import os

import StringIO
import base64
import csv
import pooler
import wizard

from osv import fields, osv, orm
from tools import ustr
from tools.translate import _

from time import strftime
from reporte_cambio_precios import main_report

import logging

logger = logging.getLogger(__name__)

LOP   = chr(27) + chr(67) + chr(18)  # fija largo de página en ¿ 12 líneas ?
LFCR  = chr(10) + chr(13)            # line feed + carriage return
FF    = chr(12)                      # form feed - new page
RESET = chr(27) + chr(60)            # reset printer
RST   = chr(27) + '!\x00'            # equivalente

PRINTER  = "LX-350"
path2pos = "/admi/mantenimiento/"
path2rep = "/tmp/"
path2bak = "/admi/erp/backup/al_pos/"

prefix = 'precios_'



class ip_export_product(osv.osv_memory):
    """Export Product to POS"""
    _name = 'ip.export.product'
    _description = 'Export Product to POS'

    def dummy_all(self):
        pass

    _columns = {
        'data': fields.binary('Size', readonly=True),
        'name': fields.char('Filename', 64, readonly=True),
        'mode': fields.selection([('master','Maestro'), ('batch','ABM'), ('online','En línea')], string="Modo", help='Maestro - Envía todos los artículos, tandem y equivalencias. No procesa menos de 10000 líneas.\nABM     - Altas, Bajas y Modificaciones. (actualizar precios)\nOnline - Actualizaciones de menos de 300 artículos. Directo a las cajas. La caja lo recibe cuando esté libre.'),
        'report_prices': fields.boolean("Print updated prices", help="Marcar para imprimir el listado de cambios de precio"),
    }

    _defaults = {
        'mode': 'online',
        'report_prices': False
    }

    def _write_to(self, fname, data):
        with open(fname, "w") as file_obj:
            file_obj.write(data)


    def export_data(self,cr,uid,ids,context=None):

        context = context or {}

        # batch activo ?
        master_enabled = True
        batch_enabled = True
        online_enabled = True

        prod_ids  = []      # articulos afectados
        ean_ids   = []      # equivs ean13 afectados

        # primer registro del wizard
        wiz = self.browse(cr, uid, ids)[0]

        subfix = strftime("%0m%0d%0H%0M")

        if wiz.mode == 'master':
            if not master_enabled:
                raise osv.except_osv(_("Export Product to POS error"), _("Impossible to export the produtcs. Be sure \"Master\" mode is enable."))
            filename = path2pos + 'bo_batch.all'
            fileflag = path2pos + 'bo_batch.afl'
            filebak  = path2bak + 'bo_batch_' + subfix + '.all'

        elif wiz.mode == 'batch':
            if not batch_enabled:
                raise osv.except_osv(_("Export Product to POS error"), _("Impossible to export the produtcs. Be sure \"Batch\" mode is enable."))
            filename = path2pos + 'bo_batch.txt'
            fileflag = path2pos + 'bo_batch.tfl'
            filebak  = path2bak + 'bo_batch_' + subfix + '.txt'

        elif wiz.mode == 'online':
            if not online_enabled:
                raise osv.except_osv(_("Export Product to POS error"), _("Impossible to export the produtcs. Be sure \"On line\" mode is enable."))
            filename = path2pos + 'bo_online.txt'
            fileflag = path2pos + 'bo_online.tfl'
            filebak  = path2bak + 'bo_online_' + subfix + '.txt'

        """ Prepara objetos para las colecciones de elementos a exportar"""
        # para artículos marcado a insertar/actualizar
        file_txt = StringIO.StringIO()
        writer = csv.writer(file_txt, delimiter= ':', lineterminator='\r\n')

        #   para tandem
        file_ta = StringIO.StringIO()
        ta_writer = csv.writer(file_ta, delimiter= ':', lineterminator='\r\n')

        #   para equivalencias
        file_eq = StringIO.StringIO()
        eq_writer = csv.writer(file_eq, delimiter= ':', lineterminator='\r\n')
        # para artículos a lista de precios
        pre_file = StringIO.StringIO()
        pre_writer = csv.writer(pre_file, delimiter= '|', lineterminator='\n')


        """ Nombres de archivos de salida """
        # nombre de archivo de salida del reporte de cambio de precios
        report_filename = path2rep + prefix + subfix + '.txt'
        report_backup   = path2bak + prefix + subfix + '.bak'
        print_rep_fname = path2rep + prefix + subfix + '.prn'



        """Armar colecciones de articulos (ids) para enviar al pos"""
        # instancias de objetos con acceso a los articulos y cod.equivalentes
        product_obj = pooler.get_pool(cr.dbname).get('product.product')
        ean_obj = pooler.get_pool(cr.dbname).get('product.ean13')

        prod_ids  = []      # articulos afectados
        ean_ids   = []      # equivs ean13 afectados

        if wiz.mode == 'master':
            prod_ids = product_obj.search(cr, uid, [])
            ean_ids = ean_obj.search(cr, uid, [])
        else:
            prod_ids = product_obj.search(cr, uid, [ ('pos_action','!=','N') ] )
            ean_ids = ean_obj.search(cr, uid, [ ('ean_action','!=','N') ] )

        if ean_ids:
            equiv_list = []
            for ean in ean_obj.browse(cr, uid, ean_ids):
                eline = []
                if ean.ean_action not in 'D':
                    eline.append('I')
                    eline.append('ARTICULOS_EQUIVALENTES')
                    eline.append(ean.name)
                    eline.append(ean.product_id.cod_sto)
                else:
                    eline.append('D')
                    eline.append('ARTICULOS_EQUIVALENTES')
                    eline.append(ean.name)
                    # hay que marcar estos ids para que el erp en lugar de volver
                    # la bandera a 'N' lod de de baja
                equiv_list.append(eline)

            for eq_row in equiv_list:
                eq_writer.writerow([ s for s in eq_row])

        if prod_ids == [] and ean_ids == []:
                raise osv.except_osv('Error', _('We cannot find any product to send !'))

        # no queremos duplicados
        prod_ids = list(set(prod_ids))

        # Itera sobre los objetos correspondientes a los ids recolectados
        # de "product.product"

        # TODO: desinfeccion. hay campos/datos que no exixten en base de datos
        # dado que el modulo no son not null iva_sto, por ejemplo.
        #####################################################################
        #####################################################################

        for prod in product_obj.browse(cr, uid, prod_ids):

            row = []        # eviar al pos
            row_pre = []    # imprimir lista de precios modificados.

            # Accion
            if not prod.in_pos and prod.pos_action is not 'D':
                row.append('I')
            else:
                row.append(prod.pos_action)

            # Entidad
            row.append('ARTICULOS')

            # CodigoArticulo
            row.append(prod.cod_sto)
            row_pre.append(prod.cod_sto)

            # Descripcion
            soyuni = 0
            if isinstance(prod.description_sale, unicode):
                name = prod.description_sale.encode('iso-8859-1')
                soyuni += 1
            else:
                name = prod.description_sale

            name = name.replace(':','-')
            name = name.replace(' pre','')
            name = name.replace(' pre env','')
            name = name.replace('/','').strip()

            row.append( name )
            row_pre.append( name )
            # CodigoEntorno
            row.append('0')

            # CodigoSubFamilia
            categ = str(prod.categ_id.id) if prod.categ_id.id > 20 else '20'
            row.append(categ)

            # CodigoMoneda
            #row.append('UYU')
            row.append('1')

            # CodigoIVA

            iva = '3'
            if int(prod.iva_sto.strip()) == 2:
                iva = '2'
            else:
                iva = '3' if int(prod.iva_sto.strip()) == 1 else '1'

            row.append(iva)

            # PrecioArticulo
            row.append('{:.2f}'.format(prod.list_price).replace('.',''))
            row_pre.append('{:.2f}'.format(prod.list_price))

            row.append(prod.atr16hex)
            # CantidadPack
            if isinstance(prod.packaging, list) and prod.packaging:
                row.append('{:.0f}'.format(prod.packaging[0].qty))
            else:
                row.append('1')

            # CodigoMedida
            ucodm = prod.uom_id.category_id.name
            if ucodm == u'Unit' or ucodm == u'Unidad':
                # '1' unidades
                row.append('1')
                row_pre.append('Ud')
            elif ucodm == u'Weight' or ucodm == u'Peso':
                #'2'  kilogramos
                row.append('2')
                row_pre.append('Kg')
            else:
                row.append('ERROR-CodigoMedida-ERROR')
                row_pre.append('-')


            # carga la linea en el objeto del archivo de intercambio
            writer.writerow([s for s in row])

            # Tandem
            if prod.envase_obligatorio:
                # retorna una linea o nada.
                # tandem = _CodigoTandem(prod.id, prod.asoc_code_id,)entidad='TANDEM')
                tand_row = ['I', 'TANDEM', prod.cod_sto, prod.asoc_code_id.cod_sto]
                ta_writer.writerow([ s for s in tand_row])

            pre_writer.writerow([r for r in row_pre])

        # Fin de proceso de linea del producto.

        equivalencias = file_eq.getvalue()
        tandems = file_ta.getvalue()

        # Codifica y copia el objeto csv en el objeto 'file'
        # prepare data file
        bo_batch_data = file_txt.getvalue()

        #agrega equivalencias y tandems
        if equivalencias:
            bo_batch_data += ''.join(equivalencias)
        if tandems:
            bo_batch_data += ''.join(tandems)

        # prepare data flag
        lineas = 0
        pre = ''
        for i in bo_batch_data:
            if i == '\r':
                pre = 'CR'
                continue
            elif pre == 'CR' and i == '\n':
                lineas += 1
                pre = ''
        flag = "%s %s \r\n" % (lineas, filename.split("/")[-1:][0])

        ## solo para documentar aquí
        # PRINTER  = "LX-350"
        # path2pos = "/admi/mantenimiento/"
        # path2rep = "/tmp/"
        # path2bak = "/admi/erp/backup/al_pos/"


        # bandera a mantenimiento
        result = False
        try:

            self._write_to(filename, bo_batch_data)
            self._write_to(fileflag, flag)
            self._write_to(filebak, bo_batch_data)
            result = True
        except:
            raise osv.except_osv(_("Export Product to POS error"), _("Impossible to write POS files."))


        if wiz.report_prices and result:

            # obtiene el string csv, lo pasa al metodo "main_report"
            data_precios = pre_file.getvalue()

            # da formato e imprime reporte
            to_file = main_report(data_precios)

            self._write_to(report_filename, to_file)
            self._write_to(report_backup, to_file)

            print "Se imprime el listado de cabio de precios \n"
            cmdprn = 'lp -d %s %s' % (PRINTER, report_filename)  # print_rep_fname) #

            salida = os.system( cmdprn )

            ## para documentar aqui
            # PRINTER  = "LX-350"
            # path2pos = "/admi/mantenimiento/"
            # path2rep = "/tmp/"
            # path2bak = "/admi/erp/backup/al_pos/"
            """ Nombres de archivos de salida """
            # report_filename = path2rep + prefix + subfix + '.txt'
            # report_backup   = path2bak + prefix + subfix + '.bak'
            # print_rep_fname = path2rep + prefix + subfix + '.prn'

        if result:
            # cambia los estados de los objetos procesados
            #

            if prod_ids:

                IDS_P =  (tuple(prod_ids),) if len(prod_ids) > 1 else '('+str(prod_ids[0])+')'

                qry_pos =  """BEGIN TRANSACTION;
                            UPDATE product_product
                            SET    pos_action = 'N'
                            WHERE id in %s ;
                            END TRANSACTION;""" % IDS_P
                try:
                    cr.execute(qry_pos)
                except:
                    raise osv.except_osv(_("Export Product to POS error"),
                                         _("Impossible to change pos state in products. This process should be reexecuted!"))
            if ean_ids:
                if isinstance(ean_ids,int):
                    IDS_E = '('+str(ean_ids)+')'
                else:
                    IDS_E = (tuple(ean_ids),) if len(ean_ids) > 1 else '('+str(ean_ids[0])+')'

                qry_ean =  """BEGIN TRANSACTION;
                            UPDATE product_ean13
                            SET    ean_action = 'N'
                            WHERE id in %s ;
                            END TRANSACTION;""" % IDS_E
                try:
                    cr.execute(qry_ean)
                except:
                    raise osv.except_osv(_("Export Product to POS error"),
                                        _("Impossible to change products pos state. This process should be reexecuted!"))

        #return result
        return {'type': 'ir.actions.act_window_close'}

ip_export_product()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
