#  -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2015 Infoprimo SL (<http://infoprimo.com.uy>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class ip_returnable_link(osv.osv_memory):

    """
        Wizard ip_returnable_link
        -------------------------

        1. Rearma los vinculos entre un articulo y el envase retornable.
           Normalmente la integridad referencial de este vinculo descanza
           en la persistencia de la db y en el orm de open. Cuando se migran
           articulos desde el sistema heredado es necesario forzar la
           reconstruccion de los viculos.
           En openerp el articulo y el envase son objeto de la misma clase
           por lo tanto viven en la misma tabla. Ambos son un objeto de la
           clase “product.product”.

        2. Asiga los atributos ALIMENTOS, TANDEM.

        3. Asiga los atributos a los articulos de BALANZA.

        4. Rearma las relaciones ARTÍCULOS/TASA de iva. #TODO:

    """

    _description = 'Relink Products / Returnables'
    _name = 'ip.returnable.link'

    def run_relink(self, cr, uid, ids, context=None):
        """
            @param self: The object pointer.
            @param cr: A database cursor
            @param uid: ID of the user currently logged in
            @param ids: List of IDs selected
            @param context: A standard dictionary
        """
        context = context or {}

        # se levanta la tabla de artículos
        product_product_obj = self.pool.get("product.product")

        # se obtienen los ids de todos los productos
        prod_ids = product_product_obj.search(cr, uid, [])

        # se obtienen ids de artículos con envase
        try:
            ids_env_obligatorio = [ i.id for i in product_product_obj.browse(cr, uid, prod_ids) if i.product_tmpl_id.cod_aso != u'0' and isinstance(i.product_tmpl_id.cod_aso, unicode) and str(i.product_tmpl_id.cod_aso).isdigit() ]
        except:
            raise osv.except_osv('Error', 'One or more "cod_aso" has worng type')


        # ## ACTIVA envase_obligatorio
        product_product_obj.write(cr, uid, ids_env_obligatorio, {'envase_obligatorio': True})
        print "Envase obligatorio"

        # ENLZAR artículos y envases
        # --------------------------
        """
            Obtener los objetos-articulos con envase """
        obj_env_obligatorio = product_product_obj.browse(cr, uid, ids_env_obligatorio)
        """
            Obtener todos los codigos de envase de los articulos con envase """
        codigo_envases = [ str(prod.cod_aso) for prod in obj_env_obligatorio ]
        """
            Crear una lista de objetos-articulo para los articulos con envase"""
        obj_envases = [ product for product in product_product_obj.browse(cr, uid,
                           prod_ids) if product.cod_sto in codigo_envases ]
        """
            Itera sobre los objetos-articulo de los articulos _con_ envase """
        for prod in obj_env_obligatorio:
            """
                Itera sobre los objetos-articulo de los articulos que _son_ envase """
            for envase in obj_envases:
                #   Código de envase del artículo _con_ envase == al código de artículo del que _es_ envase.
                if prod.cod_aso == envase.cod_sto:
                    """
                        ACTIVA el atributo "es_envase" en el objeto "envase" """
                    product_product_obj.write(cr, uid, envase.id, {'es_envase': True}, context={} )
                    """
                        ENLAZA el envase en el registro del articulo y ACTIVA el bit '09' atributo TANDEM. """
                    product_product_obj.write(cr, uid, prod.id, {'asoc_code_id':  envase.id, 'b09': True}, context={} )
                    """
                        ¡Encontramos el envase del articulo! El "break" evita
                         los restantes ciclos de iteracion de “for envase”,
                         salta al siguiente ciclo de “for prod”."""
                    break
        print "Fin Envases y marcado de articulos TANDEM"

        # SETEA atributos en artículos de balanzas "C00C000100000000"
        # ----------------------------------------------------------
        """
           1. Convierte uno a uno los 16 caracteres hexa en binario,
              se agregarn a un string vacio """
        atrib16h = 'C00C000100000000'
        bin64 = ''
        for i in range(16):
            bin64 += ''.join("{0:04b}".format(int(atrib16h[i],16)))
        """
            2.  Prepara las parejas "campo = valor" para la query SQL """
        bits = ['b01','b02','b03','b04','b05','b06','b07','b08',
                'b09','b10','b11','b12','b13','b14','b15','b16',
                'b17','b18','b19','b20','b21','b22','b23','b24',
                'b25','b26','b27','b28','b29','b30','b31','b32',
                'b33','b34','b35','b36','b37','b38','b39','b40',
                'b41','b42','b43','b44','b45','b46','b47','b48',
                'b49','b50','b51','b52','b53','b54','b55','b56',
                'b57','b58','b59','b60','b61','b62','b63','b64']
        values = ''

        for i in range(64):
            if i != 63:
                values += "%s=%s, " % (bits[i], True if int(bin64[i]) else False )
            else:
                # quita la "," en el último atrributo
                # "(... ) SET b01='b01x', (... ) b63='x63', b64='x65 WHERE ..."
                values += "%s=%s " %  (bits[i], True if int(bin64[i]) else False )
        """
            2. Crear la query  """
        qbalanzas =  """BEGIN TRANSACTION;
                        UPDATE product_product
                           SET %s""" % values
        qbalanzas +=""" WHERE product_tmpl_id
                        IN (SELECT pt.id
                              FROM product_template pt
                             WHERE CAST(pt.cod_sto AS int8) >= 2600000
                               AND CAST(pt.cod_sto as int8) <= 26999990);
                        END TRANSACTION;"""
        """
            3. Ejecutar  """
        cr.execute(qbalanzas)
        print "Fin marcado atributos de BALANZAS"

        # SETEA attrib “ALIMENTOS”
        # ------------------------
        qalimentos = """BEGIN TRANSACTION;
                  UPDATE  product_product p
                     SET b01 = True
                    FROM product_template t
                   WHERE p.product_tmpl_id = t.id
                     AND t.categ_id
                      IN ( SELECT c.id
                             FROM product_category c
                            WHERE parent_id
                               IN ( SELECT pc.id
                                      FROM product_category pc
                                     WHERE name
                                        IN ('ALIMENTOS ENVASADOS',
                                             'FIAMBRERIA ENVASADOS',
                                             'VEGETALES', 'LACTEOS',
                                             'CARNICERIA','PANADERIA',
                                             'FIAMBRERIA FRACIONADOS',
                                             'FRACIONADOS MARCA PROPIA',
                                             'ROTICERIA')
                                  )
                          );
                END TRANSACTION; """

        cr.execute(qalimentos)
        print "Fin marcado atributo ALIMENTOS"

        # Vincula Artículo/Iva
        # --------------------
        """
            TODO:  un mamaracho basado en los "id" de account_tax REAHACER
        
          select id,name,amount,price_include from account_tax order by id;
                                                  Incl/No incl
            1 | IVA Ventas (22%)     | 0.2200 |     T
            2 | IVA Ventas (10%)     | 0.1000 |     T
            3 | Ventas Exentos IVA   | 0.0000 |         F
            4 | IVA Compras (22%)    | 0.2200 |         F
            5 | IVA Compras (10%)    | 0.1000 |         F
            6 | Compras Exentos IVA  | 0.0000 |         F
            7 | Iva INC. Compras 22% | 0.2200 |     T
            8 | Iva INC. Compras 10% | 0.1000 |     T
           ¯¯\___________
                        ¯¯¯¯¯¯¯¯¯¯¯¯\
                                  | csv | dbf
           -----------------------|-----|-----
            Ventas Exentos IVA    |   3 |  1
            IVA Ventas (22%)      |   1 |  3
            IVA Ventas (10%)      |   2 |  2
                                  |     |
            IVA INC Compras (22%) |   7 |  3
            IVA INC Compras (10%) |   8 |  2
                                  |     |
            IVA Compras (22%)     |   4 |  3
            IVA Compras (10%)     |   5 |  2
            Compras Exentos IVA   |   6 |  1

             CSV  | DBF
           -------|------
            3,6   |  1
            2,5,8 |  2
            1,4,7 |  3

        acc_tax_obj = self.pool.get("account_tax")
        taxes = acc_tax_obj.search(cr,uid,[])
        for tin in taxes:
            taxacc_tax_obj.browse(cr,uid,tid"""

        """
        cr.execute(" ""BEGIN TRANSACTION;
                    TRUNCATE product_supplier_taxes_rel, product_taxes_rel;
                    INSERT INTO product_supplier_taxes_rel
                        SELECT id AS prod_id, 7 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='3';
                    INSERT INTO product_supplier_taxes_rel
                        SELECT id AS prod_id, 8 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='2';
                    INSERT INTO product_supplier_taxes_rel
                        SELECT id AS prod_id, 6 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='1';
                    INSERT INTO product_taxes_rel
                        SELECT id AS prod_id, 1 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='3';
                    INSERT INTO product_taxes_rel
                        SELECT id AS prod_id, 2 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='2';
                    INSERT INTO product_taxes_rel
                        SELECT id AS prod_id, 3 AS tax_id
                        FROM product_template
                        WHERE        iva_sto='1';
                    END TRANSACTION;"" ") """
        print "Fin reconstruccion de tasas de iva \n\t\t\t FIN de proceso."


        return {'type': 'ir.actions.act_window_close'}

ip_returnable_link()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
